function eigenvalues = run_stability_check(a,b,c,d,e,f,g,h,E0)
    run physical_parameters;
    
    fprintf('***********************************\n');
    fprintf('CHECKING STABILITY OF PERIODIC GAIT\n');
     
    %% Solving DDphi = f0 and alpha*DDtheta + beta*Dtheta^2 + gamma = 0 
    %  for variable values along target trajectory

    fprintf('****************************************************\n');
    fprintf('Computing required parameter values along trajectory: ');

    options = odeset('reltol',1e-9,'abstol',1e-9);
    step_size = 0.0001;

    % Finding phi and associated values
    [time_phi, xout] = ode45(@(theta,x) f0(theta,x,E0),...
                                a:-step_size:c, [e; f/b], options);
    phi = xout(:,1);
    Dphi= xout(:,2);

    % Removing possible NaN results
    for i=1:length(time_phi),
        if (~isnan(phi(i))) && (~isnan(Dphi(i))),
            theta_inputs_to_phi(i) = time_phi(i);
            phi_evaluated_at_theta(i) = phi(i);
            Dphi_evaluated_at_theta(i) = Dphi(i);
        else
            break
        end
    end

    theta_inputs_to_phi(1) = a;
    theta_inputs_to_phi(end) = c; 

    phi_evaluated_at_theta(1) = e;
    phi_evaluated_at_theta(end) = g;

    Dphi_evaluated_at_theta(1) = f/b;
    Dphi_evaluated_at_theta(end) = h/d;

    % Finding period of gait, theta and associated values using aby -
    % equation
    liberal_period_estimate = 2;
    options = odeset('reltol',1e-9,'abstol',1e-9,'Events',...
                        @(t,x) aby_weighted_event(t,x,c));

    [time_aby, xout] = ode45(@(t,x) aby_weighted(t,x, ...
        theta_inputs_to_phi, phi_evaluated_at_theta, ...
        Dphi_evaluated_at_theta), ...
        0:step_size:liberal_period_estimate, [a; b], options);
    theta = xout(:,1);
    Dtheta = xout(:,2);

    % Removing possible NaN results
    for i=1:length(time_aby),
        if (~isnan(theta(i))) && (~isnan(Dtheta(i)))
            time_inputs_to_theta(i) = time_aby(i); 
            theta_evaluated_at_time(i) = theta(i);
            Dtheta_evaluated_at_time(i) = Dtheta(i); 
        else
            break
        end
    end

    gait_period = time_aby(end);

    time_inputs_to_theta(1) = 0;     
    time_inputs_to_theta(end) = gait_period;

    theta_evaluated_at_time(1) = a;       
    theta_evaluated_at_time(end) = c;

    Dtheta_evaluated_at_time(1) = b;     
    Dtheta_evaluated_at_time(end) = d;

    fprintf('DONE!\n');
    fprintf('********************************************************\n');

    %% Calculating eigenvalues of transition matrix 
    % and assessing stability of perioid gait

    % Integrating linearized transversal dynamics until impact
    transverse_dynamics_shortened = @(t, zeta) ...
        transverse_linearization_continuous(t, zeta, ...
        theta_inputs_to_phi, phi_evaluated_at_theta, ...
        Dphi_evaluated_at_theta, time_inputs_to_theta, ...
        theta_evaluated_at_time, Dtheta_evaluated_at_time, E0);

    options = odeset('reltol',1e-9,'abstol',1e-9);
    fprintf('Integrating linearized transverse coordinates\n');
    fprintf('First integration: ');

    [t,zeta_n] = ode45(transverse_dynamics_shortened,...
        [0 gait_period],[1,0,0]',options); 
    zeta_n_1 = zeta_n(end,:)';

    fprintf(' DONE!\n');
    fprintf('Second integration: ');

    [t,zeta_n] = ode45(transverse_dynamics_shortened,...
        [0 gait_period],[0,1,0]',options); 
    zeta_n_2 = zeta_n(end,:)';

    fprintf('DONE!\n');
    fprintf('Third integration: ');
    
    [t,zeta_n] = ode45(transverse_dynamics_shortened,...
        [0 gait_period],[0,0,1]',options); 
    zeta_n_3 = zeta_n(end,:)';

    fprintf(' DONE!\n');
    
    % Calculating update matrix
    transverse_update_shortened = @(zeta_n) ...
        transverse_linearization_discrete(zeta_n, theta_inputs_to_phi,...
        phi_evaluated_at_theta, Dphi_evaluated_at_theta,...
        time_inputs_to_theta, theta_evaluated_at_time,...
        Dtheta_evaluated_at_time, gait_period, E0);

    update_matrix = [transverse_update_shortened(zeta_n_1), ...
        transverse_update_shortened(zeta_n_2), ...
        transverse_update_shortened(zeta_n_3)];

    % Checking eigenvalues of update matrix
    eigenvalues = sort(eig(update_matrix));

    fprintf('*****************************************************\n');
    fprintf('The found periodic gait is ');
    if max(abs(eigenvalues)) < 1
        fprintf('STABLE'); 
    else
        fprintf('UNSTABLE'); 
    end
    fprintf(' with eigenvalues\n');

    for i=1:3
        fprintf('%d. Eigenvalue = ',i);

        if real(eigenvalues(i)) >= 0
            fprintf(' ');
        end

        fprintf('%0.3f',real(eigenvalues(i)));
        if imag(eigenvalues(i)) < 0
            fprintf(' - ');
        else
            fprintf(' + ');
        end
        fprintf('%0.3fi', abs(imag(eigenvalues(i))));

        if abs(eigenvalues(i)) < 1
            fprintf('   OK!\n');
        else
            fprintf('   Outside unit circle!\n');
        end
    end
    fprintf('*****************************************************\n');
end

